import datasette
from cachetools import TTLCache
import httpx


@datasette.hookimpl
def actors_from_ids(datasette, actor_ids):
    async def inner():
        actors = await _actors_from_ids_remote(datasette, actor_ids)
        if (datasette.plugin_config("datasette-remote-actors") or {}).get(
            "datasette-profiles"
        ):
            # Extend results with extra details from "profiles" table
            db = datasette.get_internal_database()
            profiles = {
                str(row["id"]): dict(row)
                for row in (
                    await db.execute(
                        "select * from profiles where id in ({})".format(
                            ",".join("?" for _ in actor_ids)
                        ),
                        actor_ids,
                    )
                ).rows
            }
            for actor_id in actors:
                if str(actor_id) in profiles:
                    actors[actor_id].update(
                        {
                            key: value
                            for key, value in profiles[str(actor_id)].items()
                            if value is not None and key != "id"
                        }
                    )
        return actors

    return inner


async def _actors_from_ids_remote(datasette, actor_ids):
    config = datasette.plugin_config("datasette-remote-actors") or {}
    if not config.get("url"):
        return None

    cache = None
    if config.get("ttl"):
        cache = getattr(datasette, "_remote_actors_cache", None)
        if cache is None:
            cache = datasette._remote_actors_cache = TTLCache(
                maxsize=1000, ttl=config["ttl"]
            )

    to_fetch = []
    from_cache = {}
    for actor_id in actor_ids:
        actor_id = str(actor_id)
        if cache is not None and actor_id in cache:
            from_cache[actor_id] = cache[actor_id]
        else:
            to_fetch.append(actor_id)

    if not to_fetch:
        return from_cache

    token = config.get("token")
    headers = {}
    if token:
        headers["Authorization"] = f"Bearer {token}"
    async with httpx.AsyncClient() as client:
        response = await client.get(
            config["url"],
            params={"ids": ",".join(map(str, to_fetch))},
            headers=headers,
        )
    if response.status_code != 200:
        return None
    actors = response.json()
    if cache is not None:
        for actor_id, actor in actors.items():
            cache[actor_id] = actor
    return {**from_cache, **actors}



from datasette.app import Datasette
import pytest
import json

ENDPOINT_URL = "https://example.com/actors.json?ids=1"


@pytest.fixture
def non_mocked_hosts():
    return ["localhost"]


@pytest.mark.asyncio
@pytest.mark.parametrize("token", (False, True))
@pytest.mark.parametrize("ttl", (False, True))
@pytest.mark.parametrize("ids", (["1"], [1]))
@pytest.mark.parametrize("remote_id", ("1", 1))
async def test_remote_actors(httpx_mock, token, ttl, ids, remote_id):
    httpx_mock.add_response(
        url=ENDPOINT_URL,
        json={
            "1": {"id": remote_id, "name": "Cleopatra"},
        },
        is_reusable=True,
    )
    settings = {"url": ENDPOINT_URL}
    if token:
        settings["token"] = "xyz"
    if ttl:
        settings["ttl"] = 60
    datasette = Datasette(
        memory=True,
        metadata={"plugins": {"datasette-remote-actors": settings}},
    )
    actors = await datasette.actors_from_ids(ids)
    request = httpx_mock.get_request()
    assert request.url == ENDPOINT_URL
    if token:
        assert request.headers["authorization"] == "Bearer xyz"
    else:
        assert "authorization" not in request.headers
    expected = {
        "1": {"id": remote_id, "name": "Cleopatra"},
    }
    assert actors == expected
    # Run a second time
    actors = await datasette.actors_from_ids(ids)
    assert actors == expected
    # With ttl should have run another request, otherwise should not
    num_requests = len(httpx_mock.get_requests())
    if ttl:
        assert num_requests == 1
    else:
        assert num_requests == 2


@pytest.mark.asyncio
@pytest.mark.parametrize("remote_id", ("1", 1))
async def test_remote_actors_datasette_profiles(httpx_mock, remote_id):
    httpx_mock.add_response(
        url=ENDPOINT_URL,
        json={
            "1": {"id": remote_id, "name": "Cleopatra"},
        },
        is_reusable=True,
    )
    settings = {"url": ENDPOINT_URL, "datasette-profiles": True}
    datasette = Datasette(
        metadata={"plugins": {"datasette-remote-actors": settings}},
    )
    await datasette.invoke_startup()
    actors = await datasette.actors_from_ids([remote_id])
    assert actors == {"1": {"id": remote_id, "name": "Cleopatra"}}
    # Now update `profiles` table
    db = datasette.get_internal_database()
    await db.execute_write("insert into profiles (id, name) values (1, 'Cleopatra 2')")
    # Run again
    actors2 = await datasette.actors_from_ids(["1"])
    assert actors2 == {"1": {"id": remote_id, "name": "Cleopatra 2"}}
